//---------------------------------------------------------------------------
#include <vcl.h>
#include <stdio.h>
#pragma hdrstop
#include "sensor.h"
//---------------------------------------------------------------------------
char *cSensorName[]	= {	"VIA686A/B",
	   					"Winbond 83783S",
	   					"Winbond 83781D",
	   					"",
	   					"Winbond 83782D",
	   					"ITE 8712",
	   					"",
	   					"",
	   					};
#define	SENSOR_NAME_COUNT	(sizeof(cSensorName))
char *cSensorErr[]	= {	"Unknown Board",
	   					"UnSupport Sensor",
	   					"Driver Failure",
	   					};
#define	SENSOR_ERR_COUNT	(sizeof(cSensorErr))
int	LibHandle = 0;
TSEN_FUNC DllInitial, DllRelease, DllVcore, DllVio, DllP5V, DllP12V;
TSEN_FUNC DllN12V, DllN5V, DllFan1, DllFan2, DllSysTemp, DllCPUTemp;
//---------------------------------------------------------------------------
bool LoadSensorEntry(void);
int GetOrder(int);
//---------------------------------------------------------------------------
#pragma argsused
int main(int argc, char* argv[])
{
	int iDllStatus;
	int iCount;
	int iTemp;

	//-----------------------------------------------------------------------
	//load sensor.dll
	//-----------------------------------------------------------------------
	LibHandle = (int) LoadLibrary("SENSOR.DLL");
	if ( LibHandle < 32 )
	{	
		printf("Can not load sensor.dll !!\n");
        return (1);
	}
	//-----------------------------------------------------------------------
	if (LoadSensorEntry() == false)
	{	goto release;	}
	//-----------------------------------------------------------------------
	iDllStatus = (DllInitial)();
	if (iDllStatus & 0xFF00)
	{	//get error when initial
		(DllRelease)();						//release dll resource
		iDllStatus = iDllStatus >> 8;
		iCount = GetOrder(iDllStatus);
		if ((iCount >=0) && (iCount < SENSOR_ERR_COUNT))
		{	
			printf("%s\n", cSensorErr[iCount]);
			goto release;
		}
	}
	//-----------------------------------------------------------------------
	iDllStatus &= 0xFF;
	iCount = GetOrder(iDllStatus);
	printf("Sensor chip : %s\n", cSensorName[iCount]);
	iTemp = (DllVcore)();
	printf(" Vcore      : %2.2fV\n", iTemp/1000.0);
	iTemp = (DllVio)();
	printf(" Vio        : %2.2fV\n", iTemp/1000.0);
	iTemp = (DllP5V)();
	printf(" +5V        : %2.2fV\n", iTemp/1000.0);
	iTemp = (DllP12V)();
	printf(" +12V       : %2.2fV\n", iTemp/1000.0);
	iTemp = (DllN5V)();
	printf(" -5V        : %2.2fV\n", iTemp/1000.0);
	iTemp = (DllN12V)();
	printf(" -12V       : %2.2fV\n", iTemp/1000.0);

	iTemp = (DllFan1)();
	printf(" Fan 1      : %d RPM\n", iTemp);
	iTemp = (DllFan2)();
	printf(" Fan 2      : %d RPM\n", iTemp);

	iTemp = (DllSysTemp)();
	printf(" System Temperature : %d C\n", iTemp);
	iTemp = (DllCPUTemp)();
	printf(" CPU Temperature    : %d C\n", iTemp);
	//-----------------------------------------------------------------------
	(DllRelease)();							//release dll resource
	//-----------------------------------------------------------------------
release:
	//-----------------------------------------------------------------------
	//release sensor.dll
	//-----------------------------------------------------------------------
 	if (LibHandle >= 32)					//release LibHandle
	{	FreeLibrary((HINSTANCE)LibHandle);
    	LibHandle = 0;
	}
	return (0);
}
//---------------------------------------------------------------------------
bool LoadSensorEntry(void)
{
	DllInitial	= (TSEN_FUNC) GetProcAddress((HINSTANCE)LibHandle, "DLLInitial");
	DllRelease	= (TSEN_FUNC) GetProcAddress((HINSTANCE)LibHandle, "DLLRelease");
	DllVcore 	= (TSEN_FUNC) GetProcAddress((HINSTANCE)LibHandle, "Vcore");
	DllVio 		= (TSEN_FUNC) GetProcAddress((HINSTANCE)LibHandle, "Vio");
	DllP5V	 	= (TSEN_FUNC) GetProcAddress((HINSTANCE)LibHandle, "P5V");
	DllP12V		= (TSEN_FUNC) GetProcAddress((HINSTANCE)LibHandle, "P12V");
	DllN5V	 	= (TSEN_FUNC) GetProcAddress((HINSTANCE)LibHandle, "N5V");
	DllN12V		= (TSEN_FUNC) GetProcAddress((HINSTANCE)LibHandle, "N12V");
	DllFan1	 	= (TSEN_FUNC) GetProcAddress((HINSTANCE)LibHandle, "Fan1");
	DllFan2		= (TSEN_FUNC) GetProcAddress((HINSTANCE)LibHandle, "Fan2");
	DllSysTemp 	= (TSEN_FUNC) GetProcAddress((HINSTANCE)LibHandle, "SysTemp");
	DllCPUTemp	= (TSEN_FUNC) GetProcAddress((HINSTANCE)LibHandle, "CpuTemp");

	if ((DllInitial 	== NULL)||		//check DLL entry point
    	(DllRelease   	== NULL)||
	    (DllVcore 		== NULL)||
	    (DllVio 		== NULL)||
	    (DllP5V 		== NULL)||
	    (DllP12V 		== NULL)||
		(DllN5V 		== NULL)||
		(DllN12V 		== NULL)||
		(DllFan1		== NULL)||
		(DllFan2		== NULL)||
		(DllSysTemp		== NULL)||
		(DllCPUTemp		== NULL))
	{	return (false);	}
	else
	{	return (true);	}
}
//---------------------------------------------------------------------------
int GetOrder(int Data)
{
	int result = -1;
	int iLoop;
	
	for(iLoop = 0; iLoop < 8; iLoop++)
	{
		if (Data & (1 << iLoop))
		{
			result = iLoop;
			break;
		}
	}
	return (result);
}
//---------------------------------------------------------------------------
